/******************************************************************************
 * FILE PURPOSE: Defines libarary directory name using getLibs
 ******************************************************************************
 * FILE NAME: package.xs
 *
 * DESCRIPTION: This file defines the library directory name for proper build
 *              in case a different directory name for storing library files 
 *              other than "lib" is used. XDC by default assumes that the 
 *              library directory is "lib" is not sepcifically indicated by use
 *              the attributes in a file called package.xs  
 *
 * TABS: NONE
 *
 * Copyright (C) 2008 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

function getLibs(prog)
{
  /* "mangle" program build attrs into an appropriate directory name */
  if(prog.build.target.name == 'C55')
  var lib = "c55/aes_c.a55";

  if(prog.build.target.name == 'C55_large')
  var lib = "c55L/aes_c.a55L";

  if(prog.build.target.name == 'C64')
  var lib = "c64/aes_c.a64";

  if(prog.build.target.name == 'C64_big_endian')
  var lib = "c64e/aes_c.a64e";

  else if (prog.build.target.name == 'C64P' && prog.build.target.suffix == '64P')
  var lib = "c64P/aes_c.a64P;c64P/aes_cm.a64P";

  else if (prog.build.target.name == 'C64P' && prog.build.target.suffix == 'e64P')
  var lib = "ce64P/aes_c.ae64P;ce64P/aes_cm.ae64P";
  
  else if (prog.build.target.name == 'C64P_big_endian' && prog.build.target.suffix == '64Pe')
  var lib = "c64Pe/aes_c.a64Pe;c64Pe/aes_cm.a64Pe";
  
  else if (prog.build.target.name == 'C64P_big_endian' && prog.build.target.suffix == 'e64Pe')
  var lib = "ce64Pe/aes_c.ae64Pe;ce64Pe/aes_cm.ae64Pe";
  
  else if (prog.build.target.name == 'C674' && prog.build.target.suffix == '674')
  var lib = "c674/aes_c.a674;c674/aes_cm.a674";

  else if (prog.build.target.name == 'C674' && prog.build.target.suffix == 'e674')
  var lib = "ce674/aes_c.ae674;ce674/aes_cm.ae674";

  else if (prog.build.target.name == 'C674_big_endian' && prog.build.target.suffix == '674e')
  var lib = "c674e/aes_c.a674e;c674e/aes_cm.a674e";

  else if (prog.build.target.name == 'C674_big_endian' && prog.build.target.suffix == 'e674e')
  var lib = "ce674e/aes_c.ae674e;ce674e/aes_cm.ae674e";

  else if (prog.build.target.name == 'C66' && prog.build.target.suffix == '66')
  var lib = "c66/aes_c.a66;c66/aes_cm.a66";

  else if (prog.build.target.name == 'C66' && prog.build.target.suffix == 'e66')
  var lib = "ce66/aes_c.ae66;ce66/aes_cm.ae66";

  else if (prog.build.target.name == 'C66_big_endian' && prog.build.target.suffix == '66e')
  var lib = "c66e/aes_c.a66e;c66e/aes_cm.a66e";
  
  else if (prog.build.target.name == 'C66_big_endian' && prog.build.target.suffix == 'e66e')
  var lib = "ce66e/aes_c.ae66e;ce66e/aes_cm.ae66e";
  
  if(prog.build.target.name == 'TIArmv6le')
  var lib = "tiarmv6le/aes_c.a11";

  if(prog.build.target.name == 'TIArmv6be')
  var lib = "tiarmv6be/aes_c.a11e";

  if(prog.build.target.name == 'Arm11')
  var lib = "tiarmv6le/aes_c.a11";

  if(prog.build.target.name == 'Arm11_big_endian')
  var lib = "tiarmv6be/aes_c.a11e";

  if(prog.build.target.name == 'GCArmv7A')
  var lib = "gcarmv7a/aes_c.av7A";

  return (lib);
}
/* nothing past this point */
